# -*- coding: UTF-8 -*-
#   icd_cylinder_2D3D.py    kita

#!!  2D空間のカム曲線から3D空間に円筒カムをモデリングするスクリプト

# 必須ファイル
#   icd_cylinder_2D3D.py, icd_cylinder_2D3D.txt, icd_cylinder_2D3D.icd, icd_cylinder_2D3D.pdf 
#   my_module.py, my_vector.py, icd_module.py, icd_module_cam.py, my_tk.py 
#
# 履歴
# 2024.4.13     安江氏作成のicd_cylinder_2D3D.pdfをヘルプより閲覧可能とする


# ICADを操るためのおまじない
import clr
clr.AddReference('sxnet')
from sxnet import *

import tkinter as tk
from tkinter import *
from tkinter import messagebox
import my_tk as mtk
import numpy as np
from numpy import linalg as LA
import icd_module as imd
import icd_module_cam as imc
import my_vector as myv
import my_module as mmd
import os
import threading
from time import sleep
import sys
import subprocess
#sys.path.append("..")   # 親ディレクトリー 親directry 上のパス 上のフォルダー

gosa= 0.00001
pi= np.pi
my_fn= os.path.basename(__file__)       # icd_*.py
icad_path= SxSys.getDir().replace('\\','/')    # c:/ICADSX
my_save_fn= '%s/%s' % (icad_path, my_fn.replace('.py','.dat'))    # c:\icadsx\....dat
kezuri_siro= 1.0

#Editorの設定
edit_fn = 'editor.ini'
if os.path.exists(edit_fn):      # fnが存在するなら
    fp = open(edit_fn, 'r')
    for line in fp.readlines():
        if line.find('#') > -1:  # #が含まれてるなら
            continue
        line = line.rstrip()    # 行末の改行を削除
        tmp = line.split(',')
        if tmp[0] == 'editor':
            editor = tmp[1]
else:
    editor = 'notepad'


if len(sys.argv) < 2:   #引き数無し ###引数無 ###argv
    yn0= True
else:    # 引数１個 
    yn0= False

def push_help():    # 開発Versionでも同じヘルプを開く
    (fn, ext) = os.path.splitext(my_fn)
    if fn[-1] == '_':
        fn= fn[:-1]
    cmd= '%s %s.txt' % (editor, fn)
    result = subprocess.Popen(cmd,shell=True)
    cmd= 'start Acrobat %s.pdf' % fn
    print(cmd)
    result = subprocess.Popen(cmd,shell=True)


def push_exe():
    # 2D切換
    SxSys.setDim(False)     # 2D切替
    (r_out,r_in,ths,the,dth0,dx0)= get_ent()
    # 基準線取得
    while True:
        ss='カム底線を選択してください' # 入力促進窓
        (type_num, seg_part)= imd.get_mouse_pick2(SxSys.ENTMODE_SEG, ss)
        if type_num == 0:   # Go
            #set_entry2(entry_msg,'基準線の選択を終了しました')
            #break
            return

        elif type_num== SxGeom.TYPE_LINE2D: # 直線 2
            print('     カム底線が選択されました')
            geom = seg_part.getGeom()
            y0= geom.sp.y    # 始点のy座標
            v0_= vec= np.array([geom.vec.x, geom.vec.y])    # 底線Vector 
            #pe= ps+ geom.leng* np.array([geom.vec.x, geom.vec.y])    # 終点
            break
        else: 
            mtk.warning('直線以外が選択されました')
            continue

    # カム線取得(実像部品対象)
    while True:
        ss='カム線(実像部品)を選択してください' # 入力促進窓
        (type_num, seg_part)= imd.get_mouse_pick2(SxSys.ENTMODE_MULTI, ss)
        if type_num == 0:   # Go
            #set_entry2(entry_msg,'基準線の選択を終了しました')
            break            
        elif type_num== 201: # 実像部品
            (x_min, x_max)= check_cam(seg_part) # 実像カム部品をチェック
            #if x_min[0]!= None:
            if the <= ths:
                the= ths+360.0
            l_th= r_out* np.deg2rad(the-ths)    # 展開全長
            th_se= np.rad2deg(l_th/r_out)
            l_x= x_max[0]- x_min[0]
            if abs(l_x- l_th) > gosa: # 始角、終角と一致しない
                ssd= 'l_x= %.6f  l_th= %.6f' % (l_x, l_th)  
                ss= '外径とカム展開長さが一致しません。モデリングを続けますか？'
                if not mtk.question_yn(ss, ssd):
                    return
            print('     カム線が選択されました')
            p02D= np.array([x_min[0], y0])
            p12D= x_min
            p22D= np.array([x_max[0], y0])
            p32D= x_max
            break
            
        else: 
            mtk.warning('実像部品以外が選択されました')
            continue

    # 無事カム曲線を取得出来たら
    seg_list= seg_part.getEntList()
    # x座標でソート
    
    SxSys.setDim(True) #3Dに切り替え 3D切り替え
    ss= '矢視方向を少しずらしてください'
    res= mtk.information(ss)
    nn= len(seg_list); cnt= 1
    cam_list= []
    for seg in seg_list:    # 実像部品内をソート
        type_num= seg.getInf().type
        if type_num== 2: # 直線
            #底線と平行かどうか
            geom= seg.getGeom()
            v1_= np.array([geom.vec.x, geom.vec.y])    # 底線Vector 
            if myv.check_parallel2D(v0_, v1_):   # v0とv1が平行かどうかをチェック
                ps= np.array([geom.sp.x, geom.sp.y])    # 始点
                (thsl, h)= get_th_h(p02D, ps)
                pe= ps+ geom.leng* np.array([geom.vec.x, geom.vec.y])    # 終点
                (tmp, h)= get_th_h(p02D, pe)
                thel= tmp- thsl
                #print(ps, pe)
                #print(thsl, thel)
                cam= imc.cyl_modeling3(r_in, r_out, thsl, thel, h)
            else:    
                cam= make_cyl_line(seg,p02D)
            cam_list.append(cam)
            #pass
        elif type_num== 5:  # 円弧
            cam= make_cyl_arc(seg,p02D)
            cam_list.append(cam)
    
        elif type_num== 16:  # Spline
            cam= make_cyl_spline(seg,p02D)
            cam_list.append(cam)
        
        if cnt== nn:
            break

        if yn0:     # 引数なし起動
            ss= '次の要素をモデリングしていいですか'
            if mtk.question_yn(ss):
                cnt+= 1
                continue
            else:   break  
        else: # yn0== False   # 引数あり起動
            sleep(0.5)
            continue

    ss='モデリングを終了しました'
    print('\n   %s\007\n' %ss)
    mtk.information(ss)
    return


"""
def sorted_x_seg_list(seg_list):
    x_list= []; i= 0
    for seg in seg_list:
        geom= seg.getGeom()
        x= geom.sp.x
"""


# 2D空間のxy座標を3D空間の角度[rad]と高さに変換する
def get_th_h(p02D, pnt):
    (r_out,r_in,ths,the,dth0,dx0)= get_ent()
    l_th= pnt[0]- p02D[0]
    th= l_th/r_out + np.deg2rad(ths)  # [rad] 
    h= pnt[1]- p02D[1]
    #print(np.rad2deg(th), h)
    #input()
    return (th, h)  # [rad, mm]

def make_cyl_line(seg, p02D):
    (r_out,r_in,ths,the,dth0,dx0)= get_ent()
    h_list= []; th_list= []
    geom= seg.getGeom()
    sp= np.array([geom.sp.x, geom.sp.y])    # 始点
    ep= sp+ geom.leng* np.array([geom.vec.x, geom.vec.y])   # 終点
    dy= dx0* geom.vec.y/ geom.vec.x    # sp->ep 微小 y  
    if sp[0] > ep[0]:   # epが左側
        sp, ep = ep, sp
    dv= np.array([dx0, dy]) # sp->ep 微小ベクトル
    v_se= ep- sp    # sp->ep ベクトル
    h0= sp[1]- p02D[1]  # 始点でのカム高さ
    nn= int(abs(round(v_se[0]/ dx0))) # 四捨五入
    if nn >= 128:
        print('     128を超えました')
        return None 
    elif nn== 0:
        print(nn)
        (th, h)= get_th_h(p02D, sp)
        th_list.append(th)
        h_list.append(h)
    else:
        for i in range(nn):
            pnt= sp+ dv* i
            (th, h)= get_th_h(p02D, pnt)
            th_list.append(th)
            h_list.append(h)
      
    pnt= ep
    (th, h)= get_th_h(p02D, pnt)
    th_list.append(th)
    h_list.append(h)
    
    if v_chk1.get():    # カム成形 ON
        r_in= r_in-kezuri_siro; r_out= r_out+kezuri_siro
    #print(h_list)   #####
    #print(th_list)  #####
    cam= imc.cyl_modeling2(h_list, th_list, r_in, r_out)
        # 高さリスト,角度リスト[rad],Cam内外半径
    return cam


def test(h_list, th_list, r_in, r_out):
    SxSys.setDim(False) #2Dに切り替え
    i= 0
    for h in h_list:
        vx= np.array([r_out* th_list[i],0.0])
        vy= np.array([0.0, h])
        imd.p2p_line(vx, vy, '赤')
        i+= 1

# 円弧
def make_cyl_arc(seg, p02D):
    (r_out,r_in,ths,the,dth0,dx0)= get_ent()
    h_list= []; th_list= []
    geom= seg.getGeom()
    cp= np.array([geom.cp.x, geom.cp.y])
    r= geom.radius
    v0= np.array([r, 0.0]) # 基準Vector
    vs= myv.rotate2(v0, np.deg2rad(geom.sang))   # 始点Vector
    ve= myv.rotate2(v0, np.deg2rad(geom.sang+ geom.eang))   # 終点Vector
    pm= geom.eang/abs(geom.eang)
    nn= int(abs(round(geom.eang/ dth0))) # 四捨五入
    if nn >= 128:
        print('     128を超えました')
        return None
    for i in range(nn):
        th= i* np.deg2rad(dth0)
        v= myv.rotate2(vs, pm*th)
        pnt= cp+ v
        (th, h)= get_th_h(p02D, pnt)
        th_list.append(th)
        h_list.append(h)
     
    pnt= cp+ ve
    (th, h)= get_th_h(p02D, pnt)
    th_list.append(th)
    h_list.append(h)
    
    #print(len(h_list), len(th_list))
    #print(h_list)
    #print(th_list)
    #test(h_list, th_list, r_in, r_out)
    if v_chk1.get():    # カム成形 ON
        r_in= r_in-kezuri_siro; r_out= r_out+kezuri_siro
    cam= imc.cyl_modeling2(h_list, th_list, r_in, r_out)
    return cam

# Spline
def make_cyl_spline(seg, p02D):
    (r_out,r_in,ths,the,dth0,dx0)= get_ent()
    h_list= []; th_list= []
    geom= seg.getGeom()
    sp= np.array([geom.pos.x, geom.pos.y])
    nn= len(geom.vec_list)+1
    if nn >= 128:
        print('     128を超えました')
        return None
    pnt= sp
    (th, h)= get_th_h(p02D, pnt)
    th_list.append(th)
    h_list.append(h)
    for vec in geom.vec_list:
        v= np.array([vec.x, vec.y])
        pnt= sp+ v
        (th, h)= get_th_h(p02D, pnt)
        th_list.append(th)
        h_list.append(h)

    if v_chk1.get():    # カム成形 ON
        r_in= r_in-kezuri_siro; r_out= r_out+kezuri_siro
    cam= imc.cyl_modeling2(h_list, th_list, r_in, r_out)
    # 高さリスト,角度リスト[rad],Cam内外半径
    #if cam!= None:
    return cam

# cam線のチェック
# 展開長さが正しいかどうかをチェックするためカムの始点座標、終点座標を返す
def check_cam(seg_part):
    pse_list= []    # 始点、終点を合わせた座標リスト
    seg_list= seg_part.getEntList()
    for seg in seg_list:
        type_num= seg.getInf().type
        geom = seg.getGeom()
        if type_num== 2: # 直線
            sp= np.array([geom.sp.x, geom.sp.y])    # 始点
            ep= sp+ geom.leng* np.array([geom.vec.x, geom.vec.y])    # 終点
        elif type_num== 5:  # 円弧 
            cp= np.array([geom.cp.x, geom.cp.y])
            x= geom.radius* np.cos(np.deg2rad(geom.sang))
            y= geom.radius* np.sin(np.deg2rad(geom.sang))
            sp= cp+ np.array([x, y])
            x= geom.radius* np.cos(np.deg2rad(geom.sang+geom.eang))
            y= geom.radius* np.sin(np.deg2rad(geom.sang+geom.eang))
            ep= cp+ np.array([x, y])
        elif type_num== 16: # Spline
            sp= np.array([geom.pos.x, geom.pos.y])
            v_tmp= np.array([geom.vec_list[-1].x, geom.vec_list[-1].y])
            ep= sp+ v_tmp
        else: 
            mtk.warning('対象外の要素が含まれています。')
            return (np.array([999,999]), np.array([999,999]))

        pse_list.append(sp)
        pse_list.append(ep)
    
    # x座標でソート
    x_list= sorted(pse_list, key=lambda x: x[0])
    # y座標でソート
    #y_list= sorted(pse_list, key=lambda x: x[1])
    x_min= x_list[0]    # 最左点座標
    x_max= x_list[-1]   # 最右点座標
    return (x_min, x_max)
    




def get_ent():
    r_out= float(entry[0].get())/2
    r_in= float(entry[1].get())/2
    ths= float(entry[2].get())
    the= float(entry[3].get())
    dth0= float(entry[4].get())
    dx0= float(entry[5].get())

    #print(r_out,r_in,ths,the,dth0,dx0)
    return (r_out,r_in,ths,the,dth0,dx0)


def callback4():    #entry[4]:分割角度Focusout
    (r_out,r_in,ths,the,dth0,dx0)= get_ent()
    dx0= '%.5f' % (r_out* np.deg2rad(dth0))
    set_entry(5, str(dx0))
    return True
    

def set_data():     # my_save_fnよりEntry detaを読み込んで書き込み
    with open(my_save_fn, 'r') as fp:
        lines= fp.readlines()    
    i= 0
    for line in lines:
        line = line.rstrip()    # 行末の改行削除
        if line.find('#') > -1:  # #が含まれてるなら
            continue

        tmp = line.split('= ')
        #print(line) #!!!!
        if line.find('entry') > -1: # Entry行
            #if len(tmp) == 2: 
            set_entry(i, tmp[1])
            i+= 1

def set_entry(idx, ss): # Entryに文字列を挿入
    entry[idx].delete(0,tk.END)
    entry[idx].insert(0,ss)

def set_entry2(ent, ss): # Entryに文字列を挿入
    ent.delete(0,tk.END)
    ent.insert(0,ss)



def save_ent_data():    # Entry 設定値保存
    ss = ''
    for i in range(len(entry)):
        #print(i)
        ss = ss + 'entry[%d]= %s\n' % (i, entry[i].get())

    fp = open(my_save_fn,'w')
    fp.write(ss)
    fp.close()    

def on_closing():       # 終了時の処理
    save_ent_data()
    ss_geom = root.geometry() 
    mmd.save_geometry(save_fn_geom, ss_geom)

    #print('\n   %s を終了しました\n' % tmp_fn)
    print('\n   %s を終了しました\n' % my_fn)

    root.destroy()


#***********************************
# ボタン等の部品配置
#************************************
nl = 8
root = tk.Tk()
root.attributes("-topmost", True)    # 常に前面に表示
root.configure(bg='RoyalBlue4')
root.title(my_fn)  #タイトル
save_fn_geom = my_fn.replace('.py', '.ini')
ss_geom = mmd.read_geometry(save_fn_geom)
if ss_geom == None:
    ss_geom = '190x%d+1000+0' % (nl*26)

root.geometry(ss_geom)

btn_bcol = 'grey21'     #ボタン背景色
btn_tcol = 'white'      #ボタン文字色
label_bcol = 'khaki3'   #ラベル背景色
label_tcol = 'black'    #ラベル文字色
e_bcol = 'ivory2'       #Entry背景色

x0= 10; y0= 2
dx= 100; dy= 22
px= 2; py= 2


# Frame1 3D作図関係  Button Grid配置
frame1 = tk.LabelFrame(root, text='Cam Spec', padx=px, pady=py,bg="RoyalBlue3",fg=btn_tcol) 
frame1.place(x=x0,y=y0)
ss_list=['外直径[mm]','内直径[mm]','始角度[deg]','終角度[deg]','分割角度[deg]','分割長さ[mm]']
#entry_txt=['220','200','90','90','1.0','3.8397']
#i= 0;
ie= 0; entry= []
for ss in ss_list:
    label= tk.Label(frame1,text= ss_list[ie], bg=label_bcol,fg=label_tcol)    
    label.grid(column=0,row=ie, padx=px, pady=py)
    entry.append(tk.Entry(frame1, width=12,justify="right",bg=e_bcol))
    #entry[ie].insert(0,str(entry_txt[ie])) 
    entry[ie].grid(column=1,row=ie, padx=px, pady=py); ie += 1
    #i+= 1

# Focusoutで分割長さを計算
sv0 = tk.StringVar()
entry[0].configure(textvariable=sv0, validate="focusout", validatecommand=callback4)
sv4 = tk.StringVar()
entry[4].configure(textvariable=sv4, validate="focusout", validatecommand=callback4)
entry[5].configure(bg= 'pink')


#---------------------------------------------------
ss = ' 実  行  ';    #実行Buttn
#---------------------------------------------------
#btn= tk.Button(root,text = ss,command = push_start,bg=btn_bcol,fg=btn_tcol) #ボタンの作成
btn= tk.Button(root,text = ss,command = push_exe,bg=btn_bcol,fg=btn_tcol) #exeボタンの作成
#btn.pack(padx= px, pady= py, side = tk.LEFT)
btn.place(x=130,y=180)

# カム成形Check Button
v_chk1= tk.BooleanVar()   # 論理値型の変数 Instance
v_chk1.set(False)         # 初期値 0
#v_chk1.set(True)         # 初期値 1
chkb_1= tk.Checkbutton(root,text= 'カム成形', variable= v_chk1)
chkb_1.place(x=50,y=180)
chkb_1['state'] = tk.DISABLED  # 無効化

# ヘルプ Button
btn_help= tk.Button(root,text = 'ヘルプ',command = push_help,bg=btn_bcol,fg=btn_tcol) 
btn_help.place(x=0,y=180)

#--------------------------------
#   起動時の処理
#--------------------------------
# icd_tenkaizu.datがあれば読み込んでパラメータセット
if os.path.exists(my_save_fn) == True:
    set_data()
else:
    entry_txt=['220','200','90','90','1.0','3.8397']
    i= 0
    for ss in entry_txt:
        set_entry(i, entry_txt[i])
        i+= 1

root.protocol("WM_DELETE_WINDOW", on_closing)
root.mainloop()
